vassign(.(sb, dlong, dwide, timing, tdiff), unpack.tversky(
     "rickrack.sqlite", include.incomplete = F))

itf = with(
    dcast(
        na.omit(data.frame(stringsAsFactors = F,
            s = dlong$s,
            str_match(char(dlong$k),
                "^itf_(near|far)([123])_(ssr|llr|choice|delaydiff)\\.(\\d+)$")[,-1],
            v = dlong$v)),
        value.var = "v",
        s + X1 + X2 + X4 ~ X3),
    data.frame(
        s,
        type = factor(X1, levels = qw(near, far)),
        round = factor(num(X2), labels = qw(round1, round2, round3)),
        tn = num(X4),
        delaydiff = num(delaydiff),
        ssr = num(ssr),
        llr = num(llr),
        choice = factor(choice, levels = qw(ss, ll))))
itf = ordf(itf, s, type, round, delaydiff)

itf.scores = ddply(itf, .(s, type, round), function(slice)
   {choices = slice$choice[order(slice$delaydiff)]
    consistencies = c(
        sum(choices == "ss"),
        sapply(seq_along(choices), function(i)
            sum(head(choices, i-1) == "ll") +
            sum(tail(choices, -i) == "ss")),
        sum(choices == "ll"))
    c(patience = -2 + 2 * mean(which(consistencies == max(consistencies))))})

itb = with(
    dcast(
        na.omit(data.frame(stringsAsFactors = F,
            s = dlong$s,
            str_match(char(dlong$k),
                "^itb_(near|far)([123])_(ssr|llr|choice|discount)\\.(\\d+)$")[,-1],
            v = dlong$v)),
        value.var = "v",
        s + X1 + X2 + X4 ~ X3),
    data.frame(
        s,
        type = factor(X1, levels = qw(near, far)),
        round = factor(num(X2), labels = qw(round1, round2, round3)),
        tn = num(X4),
        disc = num(discount),
        ssr = num(ssr),
        llr = num(llr),
        choice = factor(choice, levels = qw(ss, ll))))
itb = merge(itb,
    transform(
        na.omit(data.frame(stringsAsFactors = F,
            s = dlong$s,
            str_match(char(dlong$k),
                "^itb_(near|far)([123])_catch_(ss|ll)$")[,-1],
            v = dlong$v)),
        X2 = factor(num(X2), labels = qw(round1, round2, round3))),
    by.x = qw(s, type, round, tn),
    by.y = qw(s, X1, X2, v),
    all.x = T)
itb = cbind(
    ss(itb, select = -c(X3, choice)),
    catchtype = factor(itb$X3, levels = qw(ss, ll)),
    itb["choice"])
itb = ordf(itb, s, type, round, tn)

# Remove the rows with the final discount factors (which don't
# represent real trials) and put them in a separate data frame.
itb.discs = ss(itb, is.na(ssr))[qw(s, type, round, disc)]
row.names(itb.discs) = c()
itb = ss(itb, !is.na(ssr))

# itb.discs10 is like itb.discs, but uses the discount factor from only
# 10 non-catch trials.
itb.discs10 = ddply(ss(itb, is.na(catchtype)), .(s, type, round),
    function(slice) slice[10,])[qw(s, type, round, disc)]
row.names(itb.discs10) = c()

itm = with(
    dcast(
        na.omit(data.frame(stringsAsFactors = F,
            s = dlong$s,
            str_match(char(dlong$k),
                "^itm_(near|far)([123])_(ssr|response)\\.(\\d+)$")[,-1],
            v = dlong$v)),
        value.var = "v",
        s + X1 + X2 + X4 ~ X3),
    data.frame(
        s,
        type = factor(X1, levels = qw(near, far)),
        round = factor(num(X2), labels = qw(round1, round2, round3)),
        tn = num(X4),
        ssr = num(ssr),
        equiv = num(response)))
itm = ordf(itm, s, type, round, tn)

itm.discs = aggregate(
    ssr/equiv ~ s + type + round,
    ss(itm, equiv >= ssr),
    median)
itm.discs = ordf(itm.discs, s, type, round)
names(itm.discs)[4] = "disc"

itm.discs5 = aggregate(
    ssr/equiv ~ s + type + round,
    ss(itm, equiv >= ssr),
    function(v) median(v[1:5]))
itm.discs5 = ordf(itm.discs5, s, type, round)
names(itm.discs5)[4] = "disc"

sbrn = row.names(sb)
sb = with(dwide, transform(sb,
    good = NA,
    duration.s1 = df2nv(ss(timing, k == "session1")[qw(s, d)])[sbrn]/60,
    duration.s2 = df2nv(ss(timing, k == "session2")[qw(s, d)])[sbrn]/60,
    finished.s2 = posix.ct(df2nv(ss(timing, k == "session2")[qw(s, received)])[sbrn]),
    itf.ll.s1 = with(ss(itf, round != "round3"),
        tapply(choice == "ll", s, mean))[sbrn],
    itb.caught.s1 = with(ss(itb, !is.na(catchtype) & round != "round3"),
        tapply(catchtype != choice, s, sum))[sbrn],
    itb.ll.s1 = with(ss(itb, is.na(catchtype) & round != "round3"),
        tapply(choice == "ll", s, mean))[sbrn],
    itm.backwards.s1 = with(ss(itm, round != "round3"),
        tapply(equiv < ssr, s, sum))[sbrn],
    itf.patience.near.r1 = df2nv(ss(itf.scores, type == "near" & round == "round1")[qw(s, patience)])[sbrn],
    itf.patience.far.r1 = df2nv(ss(itf.scores, type == "far" & round == "round1")[qw(s, patience)])[sbrn],
    itb.disc.near.r1 = df2nv(ss(itb.discs, type == "near" & round == "round1")[qw(s, disc)])[sbrn],
    itb.disc.far.r1 = df2nv(ss(itb.discs, type == "far" & round == "round1")[qw(s, disc)])[sbrn],
    itm.disc.near.r1 = df2nv(ss(itm.discs, type == "near" & round == "round1")[qw(s, disc)])[sbrn],
    itm.disc.far.r1 = df2nv(ss(itm.discs, type == "far" & round == "round1")[qw(s, disc)])[sbrn],
    gender = gender,
    age = num(age),
    height = num(height_m),
    height.raw = height_m.raw,
    weight = num(weight_kg),
    weight.raw = weight_kg.raw,
    bmi = NA,
    tobacco = tobacco == "Yes",
    cigpacks = num(cigarette_packs_per_week),
    gambling = num(gamble_days_per_month),
    exercise = num(exercise_hours_per_week),
    healthy.meals = num(healthy_meals),
    floss = num(floss_per_week),
    cc.latefees = num(credit_card_late_fees),
    cc.subpay = num(credit_card_subpayment),
    savings = num(income_saved)))

# Note: HIT 2XNTYMNCWYB4DOM7XT8X81JOCC4LP9 was plagued with
# performance issues because I made more simultaneous assignments
# than my server could handle. This probably accounts for some
# of the long durations.

# s0066 said she exercised 300 hours per week, although there are
# only 168 hours in a week. I emailed her and she replied "I
# intended for my response to be 300 minutes rather than 300
# hours per week."
sb["s0066", "exercise"] = 300/60

# s0108 said he smoked 140 packs of cigarettes per week. I
# emailed him and he replied "I meant to say 140 cigarettes per
# week, 7 packs".
sb["s0108", "cigpacks"] = 7

# s0277 said she smoked 70 packs per week. I emailed her
# and she replied "Yes that was a typo it should be 7".
sb["s0277", "cigpacks"] = 7

# s0319 said she smoked 20 packs per week. I emailed her
# and she replied "More like 6 packs a week or so."
sb["s0319", "cigpacks"] = 6

# s0322 said he smoked 35 packs a week. I emailed him and he
# replied "I thought Iread that as 35 cigarettes total! I am so
# sorry. I normally smoke 5 cigarettes a day."
sb["s0322", "cigpacks"] = round(5*7 / 20)

# s0320 wrote to me unprompted "my autofill put in that I floss my
# teeth 70x a week instead of 7x"
sb["s0320", "floss"] = 7

# s0346 said he flossed 70 times a week. I emailed him and he
# replied "This was a typo, I meant to say 7."
sb["s0346", "floss"] = 7

# NHANES 2011–2012 has 9,243 non-NA weights, of which 9 are 400
# pounds or more and 1 is 450 pounds or more (but it's less than
# 476 lbs.).
#
# The weight s0078 entered was insanely huge, bigger than that
# maximum, but almost exactly 220 kg. So he probably meant to say
# 220 lbs. (which is 99.790321 kg).
sb["s0078", "weight"] = 99.790321
sb = transform(sb,
#    height.ft = round(d = 1, 3.28 * height),
#    weight.lbs = round(2.2 * weight),
    bmi = weight / height^2)

bad.s = qw(
    s0040,
      # Said he smoked 100 packs of cigarettes a week.
    # The below were emailed (before 21 Feb 2014) but have not
    # replied.
    s0131, # 172 m tall
    s0330, # 300 hours of exercise per week
    s0332) # flossing 30 times per week
bad.s = union(bad.s, row.names(ss(sb, tv >= 5 & itb.caught.s1 >= 3)))
bad.s = union(bad.s, row.names(ss(sb, tv >= 5 & itm.backwards.s1 >= 3)))
bad.s.s2 = qw(
    s0041, s0140, s0119, s0318)
      # Completed session 2 in less than 3 minutes.
good.s = setdiff(row.names(sb), bad.s)
good.s.5to8 = intersect(good.s, row.names(ss(sb, tv %in% 5:8)))
good.s.ag = setdiff(good.s.5to8, union(
    row.names(ss(sb, tv >= 5 & itb.caught.s1 >= 1)),
    row.names(ss(sb, tv >= 5 & itm.backwards.s1 >= 1))))
  # "ag" is for "aggressive", because this is a more aggressive
  # exclusion policy.
good.s.s2 = intersect(setdiff(good.s, bad.s.s2),
    row.names(ss(sb, !is.na(duration.s2))))
sb$good = row.names(sb) %in% good.s
sb$good.s2 = row.names(sb) %in% good.s.s2

criterion.vs = qw(bmi, tobacco, cigpacks, gambling, exercise,
    healthy.meals, floss, cc.latefees, cc.subpay, savings)
