# Be sure to first comment out in default.R:
# - Every line containing " <- cut("
# - The rows of asterisks
# - 'summary(new_data)'

fetch.nlsy.data = function()
   {was.at = getwd()
    setwd("nlsy79-export")
    source("default.R", local = T)
    setwd(was.at)

    rnums = colnames(new_data)
    x = qnames(vallabels(new_data))
    colnames(x) = gsub("[^0-9a-zA-Z]+", ".", tolower(colnames(x)))
    names(rnums) = colnames(x)
    varlabels = tolower(varlabels)
    names(varlabels) = colnames(x)

    # Omit subjects who didn't respond to one of the impatience questions.
    x = subset(x,
       !is.na(impatience.1.2006) &
       !is.na(impatience.2.2006))
#       impatience.1.2006 != 0 &
#       impatience_2_2006 != 0)

    # Omit constant or empty variables.
    x = x[, mapcols(x, function(v) length(unique(v[!is.na(v)])) > 1)]

    # For each question asked in more than one survey year, take
    # only the question asked closest to the year the impatience
    # questions were asked. (Ties are broken by choosing the later
    # survey year.)
    impatience.year = 2006
    qbases = unique(sub("\\.....$", "", colnames(x)))
    new.qs = c(recursive = T, lapply(qbases, function(qbase)
       {qs = colnames(x)[
            qbase == substr(colnames(x), 1, nchar(qbase)) &
            nchar(colnames(x)) == nchar(qbase) + nchar(".1992")]
        if (length(qs) == 1)
            qs
        else if (qbase %in% qw(marstat.key, mfer.15, ffer.92))
          # We make an exception, and take every year's version,
          # for these marital-status and age-of-first-sex questions.
            qs
        else
           {years = sort(dec = T, num(
                substr(qs, nchar(qbase) + nchar(".") + 1, nchar(qs))))
            year = years[which.min(abs(years - impatience.year))]
            paste0(qbase, ".", year)}}))
    x = x[,new.qs]

    # Coerce some binary factors to logicals.
    for (v in colnames(x))
        if (is.factor(x[,v]))
           {if (identical(levels(x[,v]), qw(Yes, No)))
                x[,v] = x[,v] == "Yes"
            else if (identical(levels(x[,v]), qw(YES, NO)))
                x[,v] = x[,v] == "YES"
            else if (identical(levels(x[,v]), c("0: 0  CONDITION DOES NOT APPLY", "1: 1  CONDITION APPLIES")))
                x[,v] = x[,v] == "1: 1  CONDITION APPLIES"}

    rnums = rnums[colnames(x)]
    varlabels = varlabels[colnames(x)]
    list(x, rnums, varlabels)}

vassign(.(nlsy, nlsy.rnums, nlsy.varlabels),
    cached(fetch.nlsy.data()))

export.tagset = function(rnums)
# The produced file can imported into NLS Investigator.
    write.table(rnums, file = "/tmp/vars.NLSY79",
       row.names = F, col.names = F, quote = F)

exercise.freq = function(nlsy, v1, v2, v3 = NULL, v4 = NULL)
   {durations = !is.null(v3)
    d = data.frame(
       row.names = row.names(nlsy),
       freq.n = nlsy[,v1],
       freq.unit = nlsy[,v2])
    if (durations)
        d = cbind(d,
            duration.n = nlsy[,v3],
            duration.unit = nlsy[,v4])
#    okay.durations = if (!durations) T else with(d,
#       !is.na(duration.n) & duration.n > 0 &
#       !is.na(duration.unit))
#    okay.i = with(d, ifelse(
#        freq.n == 0 | freq.unit == "Unable to do this activity",
#        T,
#        !is.na(freq.n) & !is.na(freq.unit) & (if (!durations) T else
#            !is.na(duration.n) & duration.n > 0 & !is.na(duration.unit))))
#    d[-okay.i,] = NA
    d = transform(d, freq.per.year =
        ifelse(freq.n == 0 | freq.unit == "Unable to do this activity",
            0,
            freq.n * 
              c("Per day" = 365, "Per week" = 52, "Per month" = 12, "Per year" = 1)[
                  char(freq.unit)]))
    if (durations)
       {d = transform(d, min.per.year = pmin(365*24*60, ifelse(freq.per.year == 0, 0,
            freq.per.year * 
            duration.n *
            ifelse(duration.unit == "Hours", 60, 1))))}
    d}
