# -*- R -*-

library(ggplot2)

cur.round = 0
n.rounds = 20
init.points = NULL

next.cell.hitweighted = function(lo, hi, d)
   {d = rbind(c(x = lo, higher = NA), d)
    d$hits = 0
    for (i in 2 : nrow(d))
       {if (d[i, "higher"])
            d[i : nrow(d), "hits"] = d[i : nrow(d), "hits"] + 1
        else
            d[1 : (i-1), "hits"] = d[1 : (i-1), "hits"] + 1}
    d$hits = d$hits + 1
#    print(d)
    sample(0 : (nrow(d) - 1), 1, prob = d$hits)}

next.cell.cuhit = function(lo, hi, d)
   {d = rbind(c(x = lo, higher = NA), d)
    d$hits = 0
    for (i in 2 : nrow(d))
       {if (d[i, "higher"])
            d[i : nrow(d), "hits"] = d[i : nrow(d), "hits"] + 1
        else
            d[1 : (i-1), "hits"] = d[1 : (i-1), "hits"] + 1}
    d$hits = d$hits^3 + 1
#    print(d)
    sample(0 : (nrow(d) - 1), 1, prob = d$hits)}

next.cell.rand = function(lo, hi, d)
    sample(0 : (nrow(d) - 1), 1)

next.cell.splithalf = function(lo, hi, d)
   {if (cur.round < n.rounds/2)
        best.cell(lo, hi, d)
    else
        best.cell(lo, hi, d[setdiff(rownames(d), paste0("x", 1 : n.rounds/2)),])}

worst.cell = function(lo, hi, d)
   {d = rbind(c(x = lo, higher = NA), d)
    d$hits = 0
    for (i in 2 : nrow(d))
       {if (d[i, "higher"])
            d[i : nrow(d), "hits"] = d[i : nrow(d), "hits"] + 1
        else
            d[1 : (i-1), "hits"] = d[1 : (i-1), "hits"] + 1}
    which.min(d$hits) - 1}

best.cell = function(lo, hi, d)
# lo
#
# d[1,"x"]
#
# d[2,"x"]
#
# d[3,"x"]
#
# hi   ( ~ d[4,])
   {d = rbind(c(x = lo, higher = NA), d)
    d$hits = 0
    for (i in 2 : nrow(d))
       {if (d[i, "higher"])
            d[i : nrow(d), "hits"] = d[i : nrow(d), "hits"] + 1
        else
            d[1 : (i-1), "hits"] = d[1 : (i-1), "hits"] + 1}
#    print(d)
#    message("")
    which.max(d$hits) - 1}

ilogit = function(v) 1 / (1 + exp(-v))

f = function(x)
   !rbinom(length(x), 1, ilogit(10*20*x - 10*15))
#  !rbinom(length(x), 1, ilogit(20*x - 5))
#   !rbinom(length(x), 1, ilogit(2*x - 1))

go = function(next.cell.f)
   {lo = 0
    hi = 1
    d = data.frame(
        x = c(.2, 1/2, .8),
        higher = f(c(.2, 1/2, .8)))
    init.points <<- nrow(d)
    for (round in 1 : n.rounds)
       {cur.round <<- round
        i = next.cell.f(lo, hi, d)
        x = mean(c(
            (if (i == 0) lo else d[i, "x"]),
            (if (i == nrow(d)) hi else d[i+1, "x"])))
#        message("Selected ", x)
#        message("")
        d = rbind(
           (if (i == 0) NULL else d[1 : i,]),
           data.frame(row.names = paste0("x", round), x = x, higher = f(x)),
           (if (i == nrow(d)) NULL else d[(i+1) : nrow(d),]))}
    i = best.cell(lo, hi, d)
    mean(c(
            (if (i == 0) lo else d[i, "x"]),
            (if (i == nrow(d)) hi else d[i+1, "x"])))}

# A density matrix should satisfy
#   identical(rownames(m), c("x", "density"))  &&
#   nrow(m) > 0 &&
#   m[1, "x"] == lo &&
#   diff(c(m[,"x"], hi)) %*% m[,"density"] == 1

plot.dm = function(lo, hi, dm)
  {d = rbind(dm,
       cbind(dm[2 : nrow(dm), "x"] - 1e-12,
           dm[1 : (nrow(dm) - 1),"density"]),
       c(hi, dm[nrow(dm), "density"]))
   ggplot(data.frame(d)) +
       geom_line(aes(x, density))}

median.dm = function(lo, hi, dm)
# Finds the median of a density matrix.
   {mass = 0
    for (r in 1 : nrow(dm))
       {newlen =
            (if (r == nrow(dm)) hi else dm[r + 1, "x"]) - dm[r, "x"]
        newmass = dm[r, "density"] * newlen
        if (mass + newmass >= .5)
           return(unname(dm[r, "x"] + newlen * (.5 - mass)/newmass))
        mass = mass + newmass}}

go.pba = function(pc = .75)
# Uses the original PBA (p. 14): Waeber, R. (2013). Probabilistic bisection search for stochastic root-finding (PhD thesis). Cornell University. Retrieved from http://people.orie.cornell.edu/shane/theses/ThesisRolfWaeber.pdf
   {lo = 0
    hi = 1
    dm = matrix(nrow = 1, c(lo, 1))
    colnames(dm) = c("x", "density")
#    par(ask = T)
    for (round in 1 : n.rounds)
       {med = median.dm(lo, hi, dm)
        higher = f(med)
#        message("med: ", med, "; higher: ", higher)
        i = local(
           {w = which(dm[,"x"] > med)
            if (length(w)) w[1] - 1 else nrow(dm)})
#        message("i: ", i)
        dm = rbind(
            dm[1 : i,],
            c(med, dm[i, "density"]),
            (if (i == nrow(dm)) NULL else dm[(i + 1) : nrow(dm),]))
        dm[,"density"] = dm[,"density"] * 2 *
            ifelse((dm[,"x"] >= med) == higher, pc, 1 - pc)
#        print(dm)
#        message("total mass: ", (diff(c(dm[,"x"], 1)) %*% dm[,"density"]))
#        print(plot.dm(lo, hi, dm))}
        }
    median.dm(lo, hi, dm)}
